package
Version:
v0.9.4
Opens a new window with list of versions in this module.
Published: Sep 19, 2023
License: MIT
Opens a new window with license information.
Imports: 4
Opens a new window with list of imports.
Imported by: 0
Opens a new window with list of known importers.
Documentation
¶
func GetSize() (col, row int, err error)
created with assistance of ChatGPT v4
Source Files
¶
Click to show internal directories.
Click to hide internal directories.